<?php

/*
 * EverList - A free email listserv written in PHP.
 * Copyright (C) 2008 by Kris Craig.
 * 07.20.2008
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * EverList Mail Relay
 *
 * This is a cronjob-activated script that is not designed for direct user 
 * access.  This relay manages and sends outgoing digest emails to users and
 * processes all emails in the EverList user inbox (via IMAP) according to 
 * the following standards:
 *
 * Reply-to email relaying.
 *	->Recipient address must be in the format "everlist-user#@your-domain.com" 
 * 	where "#" is the recipient's userid.
 * 	->Any subject that contains the string "AutoReply" will be automatically 
 *	rejected.
 * New posts.
 *	->Recipient address must be in the format "<forum-name>@your-domain.com".
 *	->Sender email address must be associated with an account that has posting 
 *	privileges to that forum.
 *	->Subject line must be the name of the topic ("RE:" is ignored).  If a topic 
 *	by that name doesn't exist, then a new topic by that name is automatically 
 *	created.
 *
 * All processed emails are immediately deleted from the inbox.
 *
 * --Kris
 */

require( "config.php" );
require( "common.php" );

foreach ( $_GET as $getvar => $getval )
{
	$$getvar = $getval;
}

$hostname = "kriscraig.com";
$imapport = "143";
$user = "everlist";
$pass = "555555";

$mailbox = imap_open( "{" . $hostname . ":" . $imapport . "}INBOX", "$user", "$pass" );

$nummsg = imap_num_msg( $mailbox );

$msghistory = array();
for ( $mloop = 1; $mloop <= $nummsg; $mloop++ )
{
	$msgheader = imap_headerinfo( $mailbox, $mloop );
	
	imap_delete( $mailbox, $mloop );
	
	foreach ( $msgheader->from as $fromkey => $fromobj )
	{
		$frombox = $fromobj->mailbox;
		$fromhost = $fromobj->host;
	}
	
	$replyto = $frombox . "@" . $fromhost;
	
	$replytoadr = $replyto;
	
	$fromuserdata = array();
	$fromuserdata = LoadUserByEmail( $replyto );
	
	$tostring = $msgheader->toaddress;
	
	$ccstring = $msgheader->ccaddress;
	
	$subject = $msgheader->subject;
	
	$findstring = array();
	$findstring[1] = "&quot;";
	
	$replacestring = array();
	$replacestring[1] = "\"";
	
	$subject = str_replace( $findstring, $replacestring, $subject );
	
	/* Unauthorized sender.  --Kris */
	if ( $fromuserdata["success"] == FALSE )
	{
		if ( strcasecmp( substr( $tostring, 0, 13 ), "everlist-user" ) == 0 
			|| strcasecmp( substr( $ccstring, 0, 13 ), "everlist-user" ) == 0 )
		{
			$subject = "RE: [Unauthorized Sender] $subject";
			$msgbody = "We're sorry, but your email to one of our users could not be ";
			$msgbody .= "relayed to that user because the email address you sent it ";
			$msgbody .= "from is not registered to any of our active users.  In order ";
			$msgbody .= "to minimize spam, only registered email addresses may send ";
			$msgbody .= "messages to other EverList users.";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
		}
		
		continue;
	}
	
	$fromprofiledata = array();
	$fromprofiledata = LoadProfile( $fromuserdata["userid"] );
	
	$bodystruct = imap_fetchstructure( $mailbox, $mloop );
	
	$replyto = $fromprofiledata["fullname"] . " <$replyto>";
	
	/* Uncomment below for DEBUG.  --Kris */
	/*
	print "DEBUG - $tostring - $ccstring<br />";
	print_r( $msgheader );
	die();
	*/
	
	/* Filter out those annoying "Out of Office" AutoReplies.  --Kris */
	if ( stristr( $subject, "AutoReply" ) != FALSE )
	{
		continue;
	}
	
	$msgbody = imap_body( $mailbox, $mloop );
	
	/* Skip duplicate emails (typically from CC's).  --Kris */
	$dupfound = FALSE;
	foreach ( $msghistory as $mkey => $mval )
	{
		if ( strcasecmp( $msghistory[$mkey]["replyto"], $replyto ) == 0 
			&& strcasecmp( $msghistory[$mkey]["tostring"], $tostring ) == 0 
			&& strcasecmp( $msghistory[$mkey]["subject"], $subject ) == 0 
			&& strcasecmp( $msghistory[$mkey]["msgbody"], $msgbody ) == 0 )
		{
			$dupfound = TRUE;
			break;
		}
	}
	if ( $dupfound == TRUE )
	{
		continue;
	}
	
	$msghistory[$mloop] = array();
	$msghistory[$mloop]["replyto"] = $replyto;
	$msghistory[$mloop]["tostring"] = $tostring;
	$msghistory[$mloop]["subject"] = $subject;
	$msghistory[$mloop]["msgbody"] = $msgbody;
	
	/* Message relay.  --Kris */
	if ( strcasecmp( substr( $tostring, 0, 13 ), "everlist-user" ) == 0 
		|| strcasecmp( substr( $ccstring, 0, 13 ), "everlist-user" ) == 0 )
	{
		$touserid = NULL;
		for ( $parseloop = 13; $parseloop < strlen( $tostring ); $parseloop++ )
		{
			if ( strcmp( $tostring{$parseloop}, "@" ) == 0 )
			{
				break;
			}
			else
			{
				$touserid .= $tostring{$parseloop};
			}
		}
		
		if ( $touserid == NULL || !is_numeric( $touserid ) )
		{
			/* Bad message.  --Kris */
			
			continue;
		}
		
		$userdata = array();
		$userdata = LoadUser( $touserid );
		if ( $userdata["success"] == FALSE )
		{
			/* Invalid userid.  --Kris */
			
			continue;
		}
		
		/*
		 * I'm not sure what the best way to detect the MIME text type 
		 * in the email is, so for now I just have it looking for common 
		 * HTML tags (not just "<" or ">" since they aren't necessarily 
		 * part of HTML tags).  If you know of a better way to detect the 
		 * type directly from the mailheaders, by all means replace this 
		 * little hackjob of mine with that!
		 *
		 * --Kris
		 */
		if ( stristr( $msgbody, "<br>" ) == FALSE 
			&& stristr( $msgbody, "<br />" ) == FALSE 
			&& stristr( $msgbody, "<br/>" ) == FALSE 
			&& stristr( $msgbody, "<a " ) == FALSE 
			&& stristr( $msgbody, "<img " ) == FALSE 
			&& stristr( $msgbody, "<span " ) == FALSE 
			&& stristr( $msgbody, "<div " ) == FALSE 
			&& stristr( $msgbody, "<font " ) == FALSE 
			&& stristr( $msgbody, "<p " ) == FALSE 
			&& stristr( $msgbody, "<p>" ) == FALSE 
			&& stristr( $msgbody, "<script " ) == FALSE 
			&& stristr( $msgbody, "<table" ) == FALSE )
		{
			$contenttype = "text/plain";
		}
		else
		{
			$msgbody = imap_fetchbody( $mailbox, $mloop, 2 );
			
			if ( strcasecmp( $fromhost, "hotmail.com" ) == 0 
				|| strcasecmp( $fromhost, "msn.com" ) == 0 
				|| stristr( $msgbody, "=3B" ) 
				|| stristr( $msgbody, "=3D" ) 
				|| stristr( $msgbody, "=2C" ) )
			{
				$msgbody = ParseHotmail( $msgbody );
			}
			
			$contenttype = "text/html";
		}
		
		if ( strcasecmp( $userdata["email"], $replytoadr ) )
		{
			$mailresult = mail( $userdata["email"], $subject, $msgbody, 
				// Email header
				"From: " . $replyto . "\r\n" 
				. "Reply-To: " . $replyto . "\r\n" 
				. "Content-type: $contenttype\r\n" 
				. "X-mailer: PHP/" . phpversion() );
		}
	}
	
	if ( strcasecmp( substr( $tostring, 0, 13 ), "everlist-user" ) 
		|| ( strcasecmp( substr( $ccstring, 0, 13 ), "everlist-user" ) 
			&& $ccstring != NULL ) )
	/* New Post.  --Kris */
	{
		if ( strcasecmp( substr( $ccstring, 0, 13 ), "everlist-user" ) 
			&& $ccstring != NULL )
		{
			$tostring = $ccstring;
		}
		
		if ( strcasecmp( substr( trim( $subject ), 0, 3 ), "RE:" ) == 0 )
		{
			$subject = substr( trim( $subject ), 3, strlen( $subject ) - 3 );
		}
		
		if ( strcmp( substr( trim( $subject ), 0, 1 ), "[" ) == 0 )
		{
			$newsubject = NULL;
			for ( $parseloop = 0; $parseloop < strlen( $subject ); $parseloop++ )
			{
				if ( strcmp( $subject{$parseloop}, "]" ) == 0 
					&& $newsubject == NULL )
				{
					$parseloop += 2;
					if ( $parseloop >= strlen( $subject ) )
					{
						break;
					}
					
					$newsubject .= $subject{$parseloop};
					continue;
				}
				else if ( $newsubject == NULL )
				{
					continue;
				}
				else
				{
					$newsubject .= $subject{$parseloop};
				}
			}
			
			$subject = $newsubject;
		}
		
		if ( trim( $subject == NULL ) || strlen( trim( $subject ) ) < 2 )
		{
			$msgbody = "Post subject must be at least 2 characters in length!";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		if ( strlen( $subject ) > 70 )
		{
			$msgbody = "Post subject cannot exceed 70 characters in length!";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		if ( trim( $msgbody == NULL ) || strlen( trim( $msgbody ) ) < 2 )
		{
			$msgbody = "Post body must be at least 2 characters in length!";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		if ( strlen( $msgbody ) > 204800 )
		{
			$msgbody = "Post body cannot exceed 200 KB in length!";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		$forumname = NULL;
		for ( $parseloop = 0; $parseloop < strlen( $tostring ); $parseloop++ )
		{
			if ( strcmp( $tostring{$parseloop}, "@" ) == 0 
				|| strcmp( $tostring{$parseloop}, "<" ) == 0 
				|| strcmp( $tostring{$parseloop}, ">" ) == 0 
				|| strcmp( $tostring{$parseloop}, " " ) == 0 )
			{
				break;
			}
			else if ( strcmp( $tostring{$parseloop}, "\"" ) == 0 )
			{
				continue;
			}
			else
			{
				$forumname .= $tostring{$parseloop};
			}
		}
		
		$forumdata = array();
		$forumdata = LoadForumByForumname( $forumname );
		
		if ( $forumdata["success"] == FALSE )
		{
			$msgbody = "We're sorry, but the forum '$forumname' does not exist on this listserv.";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		/* This is redundant for now, but may be useful later.  --Kris */
		if ( $fromuserdata["success"] == FALSE )
		{
			$msgbody = "Your email address is not registered on our listserv!  Only registered users may post.";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		if ( strcmp( $fromuserdata["status"], "ADMIN" ) == 0 )
		{
			$isadmin = TRUE;
		}
		else
		{
			$isadmin = FALSE;
		}
		
		if ( $forumdata["isclosed"] == 1 || $forumdata["isclosed"] == 2 
			|| ( $forumdata["isclosed"] == 3 && $isadmin == FALSE ) 
			|| ( $forumdata["isclosed"] == 4 && $isadmin == FALSE ) )
		{
			$msgbody = "This forum is presently CLOSED to new posts.  Please contact the listserv administrator for further information.";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		$subscribed = IsSubscribed( $fromprofiledata, $forumdata["fid"] );
		
		if ( $forumdata["ispublic"] != 1 && $subscribed == FALSE )
		{
			$msgbody = "You must be subscribed to post to this forum ($forumname)!";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		$limitreached = CheckPostLimitReached( $forumdata, $fromuserdata );
		
		if ( $limitreached == 1 )
		{
			$msgbody = "You have reached the maximum number of hourly posts allowed on this forum.  Please try again later.";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		else if ( $limitreached == 2 )
		{
			$msgbody = "You have reached the maximum number of daily posts allowed on this forum.  Please try again tomorrow.";
			$subject = "RE: $subject";
			
			$mailresult = mail( $replyto, $subject, $msgbody, 
				// Email header
				"From: " . $mailfrom . "\r\n" 
				. "Reply-To: " . $mailfrom . "\r\n" 
				. "Content-type: text/plain\r\n" 
				. "X-mailer: PHP/" . phpversion() );
			
			continue;
		}
		
		$subject = str_ireplace( "RE:", "", $subject );
		$subject = str_ireplace( "<", "&lt;", $subject );
		$subject = str_ireplace( ">", "&rt;", $subject );
		
		//$subjectb = trim( htmlspecialchars( addslashes( $subject ) ) );  //Not used
		$subjectb = trim( addslashes( $subject ) );
		$msgbody = trim( addslashes( $msgbody ) );
		
		$query = "select tname, tid, fid from topics where tname='$subjectb'";
		$resulttl1 = mysql_query( $query ) or die( "Query '$query' failed for resulttl1 in relaymail.php : " . mysql_error() );
		
		while ( $line = mysql_fetch_array( $resulttl1, MYSQL_ASSOC ) )
		{
			$sqloop = 0;
			foreach ( $line as $col_value )
			{
				$fieldname = mysql_field_name( $resulttl1, $sqloop );
				$$fieldname = $col_value;
				$sqloop++;
			}
		}
		
		$cur_time = time();
		
		if ( strcasecmp( $subject, $tname ) == 0 )
		{
			if ( $fid != $forumdata["fid"] )
			{
				$msgbody = "That topic ($subject) already exists on another forum!";
				$subject = "RE: $subject";
				
				$mailresult = mail( $replyto, $subject, $msgbody, 
					// Email header
					"From: " . $mailfrom . "\r\n" 
					. "Reply-To: " . $mailfrom . "\r\n" 
					. "Content-type: text/plain\r\n" 
					. "X-mailer: PHP/" . phpversion() );
				
				continue;
			}
			
			$query = "update topics set ttimestamp='$cur_time' where tname='$subjectb'";
			$resulttl4 = mysql_query( $query ) or die( "Query '$query' failed for resulttl4 in relaymail.php : " . mysql_error() );
		}
		else
		{
			$query = "insert into topics values ( NULL, '" . $forumdata["fid"] . "', ";
			$query .= "'" . $fromuserdata["userid"] . "', '$subjectb', '$cur_time', ";
			$query .= "'$ipb', 'NEW' )";
			$resulttl2 = mysql_query( $query ) or die( "Query '$query' failed for resulttl2 in relaymail.php : " . mysql_error() );
			
			$query = "select tid from topics where ttimestamp='$cur_time' AND createdbyuserid='" . $fromuserdata["userid"] . "'";
			$resulttl3 = mysql_query( $query ) or die( "Query '$query' failed for resulttl3 in relaymail.php : " . mysql_error() );
			
			while ( $line = mysql_fetch_array( $resulttl3, MYSQL_ASSOC ) )
			{
				$sqloop = 0;
				foreach ( $line as $col_value )
				{
					$fieldname = mysql_field_name( $resulttl3, $sqloop );
					$$fieldname = $col_value;
					$sqloop++;
				}
			}
			
			/* This should never happen!  --Kris */
			if ( $tid == NULL )
			{
				die( "ERROR!  Unable to determine tid for ttimestamp '$cur_time' and createdbyuserid '" . $userdata["userid"] . "'!  Please copy/paste this error into an email and send it to <a href=\"mailto: kris.craig@gmail.com\">Kris.Craig@gmail.com</a>!" );
			}
		}
		
		/*
		 * I'm not sure what the best way to detect the MIME text type 
		 * in the email is, so for now I just have it looking for common 
		 * HTML tags (not just "<" or ">" since they aren't necessarily 
		 * part of HTML tags).  If you know of a better way to detect the 
		 * type directly from the mailheaders, by all means replace this 
		 * little hackjob of mine with that!
		 *
		 * --Kris
		 */
		if ( stristr( $msgbody, "<br>" ) == FALSE 
			&& stristr( $msgbody, "<br />" ) == FALSE 
			&& stristr( $msgbody, "<br/>" ) == FALSE 
			&& stristr( $msgbody, "<a " ) == FALSE 
			&& stristr( $msgbody, "<img " ) == FALSE 
			&& stristr( $msgbody, "<span " ) == FALSE 
			&& stristr( $msgbody, "<div " ) == FALSE 
			&& stristr( $msgbody, "<font " ) == FALSE 
			&& stristr( $msgbody, "<p " ) == FALSE 
			&& stristr( $msgbody, "<p>" ) == FALSE 
			&& stristr( $msgbody, "<script " ) == FALSE 
			&& stristr( $msgbody, "<table" ) == FALSE )
		{
			$msgbody = str_replace( "\n", "<br />", $msgbody );
		}
		else
		{
			$msgbody = imap_fetchbody( $mailbox, $mloop, 2 );
			
			if ( strcasecmp( $fromhost, "hotmail.com" ) == 0 
				|| strcasecmp( $fromhost, "msn.com" ) == 0 
				|| stristr( $msgbody, "=3B" ) 
				|| stristr( $msgbody, "=3D" ) 
				|| stristr( $msgbody, "=2C" ) )
			{
				$msgbody = ParseHotmail( $msgbody );
			}
		}
		
		/* Prevent hackers from posting harmful embeds & etc.  --Kris */
		$msgbody = str_ireplace( "<script", "&lt;script", $msgbody );
		$msgbody = str_ireplace( "</script", "&lt;/script", $msgbody );
		$msgbody = str_ireplace( "<frame", "&lt;frame", $msgbody );
		$msgbody = str_ireplace( "<iframe", "&lt;iframe", $msgbody );
		$msgbody = str_ireplace( "<embed", "&lt;embed", $msgbody );
		
		$msgbody = addslashes( trim( $msgbody ) );
		
		$query = "insert into posts values ( NULL, '$tid', '" . $fromuserdata["userid"] . "', ";
		$query .= "'$msgbody', '0', '$cur_time', '$cur_time', '$ipb', 'NEW' )";
		$resulttl5 = mysql_query( $query ) or die( "Query '$query' failed for resulttl5 in relaymail.php : " . mysql_error() );
	}
}

imap_expunge( $mailbox );

imap_close( $mailbox );

/* Now send out digest email to appropriate users.  --Kris */
$cur_time = time();
$fifteenminago = 60 * 15;

$query = "select * from posts where pstatus='NEW'";
$resulttl7 = mysql_query( $query ) or die( "Query '$query' failed for resulttl7 in relaymail.php : " . mysql_error() );

$newposts = array();
$newposts[0] = 0;
while ( $line = mysql_fetch_array( $resulttl7, MYSQL_ASSOC ) )
{
	$newposts[0]++;
	$newposts[$newposts[0]] = array();
	$sqloop = 0;
	foreach ( $line as $col_value )
	{
		$fieldname = mysql_field_name( $resulttl7, $sqloop );
		$newposts[$newposts[0]][$fieldname] = $col_value;
		$sqloop++;
	}
}

$profilesdata = array();
$profilesdata = LoadProfiles();

$topicschange = NULL;
$postschange = NULL;

for ( $ploop = 1; $ploop <= $newposts[0]; $ploop++ )
{
	/* Wait at least 15 minutes so user has a chance to edit/delete.  --Kris */
	if ( $cur_time - intval( $newposts[$ploop]["ptimestamp"] ) < $fifteenminago )
	{
		continue;
	}
	
	$topicdata = array();
	$topicdata = LoadTopic( $newposts[$ploop]["tid"] );
	
	$forumdata = array();
	$forumdata = LoadForum( $topicdata["fid"] );
	
	if ( strcasecmp( $topicdata["tstatus"], "NEW" ) == 0 )
	{
		$newtopic = TRUE;
		$numreplies = 0;
		$subject = "[" . $forumdata["fname"] . "] " . $topicdata["tname"];
		
		$topicschange .= "tid='" . $topicdata["tid"] . "' OR ";
	}
	else
	{
		$newtopic = FALSE;
		
		$query = "select tid from posts where tid='" . $topicdata["tid"] . "'";
		$resulttl8 = mysql_query( $query ) or die( "Query 'query' failed for resulttl8 in messages.php : " . mysql_error() );
		
		$numreplies = -1;
		while ( $line = mysql_fetch_array( $resulttl8, MYSQL_ASSOC ) )
		{
			$numreplies++;
		}
		
		$subject = "RE: [" . $forumdata["fname"] . "] " . $topicdata["tname"];
	}
	
	$postschange .= "pid='" . $newposts[$ploop]["pid"] . "' OR ";
	
	foreach ( $profilesdata as $pskey => $profiledata )
	{
		if ( $pskey == 0 )
		{
			continue;
		}
		
		if ( !IsSubscribed( $profiledata, $topicdata["fid"] ) )
		{
			continue;
		}
		
		$digestpref = GetDigestPref( $profiledata, $topicdata["fid"] );
		
		if ( $digestpref == 0 || ( $digestpref == 2 && $newtopic == FALSE ) 
			|| ( $digestpref == 3 && $numreplies > 1 ) )
		{
			continue;
		}
		
		if ( $digestpref == 3 && $numreplies == 1 )
		{
			$replyto = $mailfrom;
			$msgbody = "There have been one or more responses to this post.<br /><br />";
			$msgbody .= "To view these responses, click the following link: ";
			$msgbody .= "<a href=\"$siteurl/viewtopic.php?tid=" . $topicdata["tid"] . "#" . $newposts[$ploop]["pid"] . "\">";
			$msgbody .= "$siteurl/viewtopic.php?tid=" . $topicdata["tid"] . "#" . $newposts[$ploop]["pid"];
			$msgbody .= "</a><br /><br /><br />";
			$msgbody .= "You will not receive anymore email notifications for replies to this thread.<br /><br />";
			$msgbody .= "Best Regards,<br /><br />";
			$msgbody .= "--EverList Mailer Robot";
		}
		else
		{
			$replyto = "everlist-user" . $newposts[$ploop]["createdbyuserid"];
			$replyto .= "@$maildomain";
			if ( $newposts[$ploop]["excludeptext"] == 1 )
			{
				$msgbody = "The author of this post has opted to omit it from the email digest, most likely due to formatting concerns.  To view the post, please click the link below.";
			}
			else
			{
				$msgbody = $newposts[$ploop]["ptext"];
			}
			
			$msgbody .= "<br /><hr width=\"100%\" /><br />Posted by:  <a href=\"$siteurl/viewprofile.php?userid=" . $profiledata["userid"] . "\">" . $profiledata["fullname"] . "</a><br />";
			$msgbody .= "Post URL: <a href=\"$siteurl/viewtopic.php?tid=" . $topicdata["tid"] . "#" . $newposts[$ploop]["pid"] . "\">";
			$msgbody .= "$siteurl/viewtopic.php?tid=" . $topicdata["tid"] . "#" . $newposts[$ploop]["pid"] . "</a>";
		}
		
		$userdata = array();
		$userdata = LoadUser( $profiledata["userid"] );
		
		$findstring = array();
		$findstring[1] = "&quot;";
		
		$replacestring = array();
		$replacestring[1] = "\"";
		
		$subject = str_replace( $findstring, $replacestring, $subject );
		
		$mailresult = mail( $userdata["email"], $subject, $msgbody, 
			// Email header
			// "CC" message is ignored; allows user to "reply to all".
			"From: " . $forumdata["fname"] . "@$maildomain" . "\r\n" 
			. "CC: " . $forumdata["fname"] . "@$maildomain" . "\r\n" 
			. "Reply-To: " . $replyto . "\r\n" 
			. "Content-type: text/html\r\n" 
			. "X-mailer: PHP/" . phpversion() );
		
		if ( $mailresult == FALSE )
		{
			die( "ERROR" );
		}
	}
}

/* Now that everything's emailed-out, let's remove the "NEW" flags.  --Kris */
$topicschange = substr( $topicschange, 0, strlen( $topicschange ) - 4 );
$postschange = substr( $postschange, 0, strlen( $postschange ) - 4 );

if ( $topicschange != NULL )
{
	$query = "update topics set tstatus='ACTIVE' where " . $topicschange;
	$resulttl9 = mysql_query( $query ) or die( "Query '$query' failed for resulttl9 : " . mysql_error() );
}

if ( $postschange != NULL )
{
	$query = "update posts set pstatus='ACTIVE' where " . $postschange;
	$resulttl10 = mysql_query( $query ) or die( "Query '$query' failed for resulttl10 : " . mysql_error() );
}
